/*
 * GImageView
 * Copyright (C) 2001 Takuro Ashie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __GIMV_IMAGE_H__
#define __GIMV_IMAGE_H__

#include "gimageview.h"


#define IMG_TYPE_NUM 13

/* definition of image format */
/* public image format */
#define IMG_TIF "TIFF"
#define IMG_JPG "JPEG"
#define IMG_PNG "PNG"
/* X Window System Image Format */
#define IMG_XBM "XBM"             /* X11 Bitmap */
#define IMG_XPM "XPM"             /* X11 Pixmap */
#define IMG_XWD "XWD"             /* X Window Dump */
/* PNM (Portable aNyMap) */
#define IMG_PPM "PPM"             /* full color */
#define IMG_PGM "PGM"             /* gray scale */
#define IMG_PBM "PBM"             /* mono */
/* IBM PC */
#define IMG_BMP "BMP"             /* Window Bitmap */
#define IMG_PCX "PCX"
/* Macintosh */
#define IMG_PICT "PICT"           /* not supported yet */
/* etc */
#define IMG_TGA  "TGA"
#define IMG_MAG  "MAG"
#define IMG_XCF  "XCF"
/* f*ck'in */
#define IMG_GIF "GIF"
#define IMG_UNKNOWN "UNKNOWN"


typedef gpointer GimvImage;


typedef enum
{
   NORMAL_MODE       = 0,
   DISSOLVE_MODE     = 1,
   MULTIPLY_MODE     = 3,
   SCREEN_MODE       = 4,
   OVERLAY_MODE      = 5,
   DIFFERENCE_MODE   = 6,
   ADDITION_MODE     = 7,
   SUBTRACT_MODE     = 8,
   DARKEN_ONLY_MODE  = 9,
   LIGHTEN_ONLY_MODE = 10,
   HUE_MODE          = 11,
   SATURATION_MODE   = 12,
   COLOR_MODE        = 13,
   VALUE_MODE        = 14
} LayerMode;


void       gimv_image_update_image_types   ();
void       gimv_image_init                 ();
GimvImage *gimv_image_load_file            (const gchar  *filename);
GimvImage *gimv_image_create_from_data     (guchar       *data,
					    gint          width,
					    gint          height,
                                            gboolean      alpha);
GimvImage *gimv_image_create_from_drawable (GdkDrawable  *drawable,
					    gint          x,
					    gint          y,
					    gint          width,
					    gint          height);
GimvImage *gimv_image_rotate_90            (GimvImage    *src_image,
					    gboolean      counter_clockwise);
GimvImage *gimv_image_rotate_180           (GimvImage    *src_image);
void       gimv_image_get_pixmap_and_mask  (GimvImage    *image,
					    GdkPixmap   **pixmap_return,
					    GdkBitmap   **mask_return);
void       gimv_image_free_pixmap_and_mask (GdkPixmap    *pixmap,
					    GdkBitmap    *mask);
GimvImage *gimv_image_scale_and_dump       (GimvImage    *src_image,
					    gint          width,
					    gint          height);
void       gimv_image_scale_get_pixmap     (GimvImage    *src_image,
					    gint          width,
					    gint          height,
					    GdkPixmap   **pixmap_return,
					    GdkBitmap   **mask_return);
void       gimv_image_get_size             (GimvImage    *image,
					    gint         *width,
					    gint         *height);
gint       gimv_image_width                (GimvImage    *image);
gint       gimv_image_height               (GimvImage    *image);
gint       gimv_image_depth                (GimvImage    *image);
gboolean   gimv_image_has_alpha            (GimvImage    *image);
gboolean   gimv_image_can_alpha            (GimvImage    *image);
gint       gimv_image_rowstride            (GimvImage    *image);
guchar    *gimv_image_get_pixels           (GimvImage    *image);
void       gimv_image_kill                 (GimvImage    *image);
gboolean   gimv_image_save_file            (GimvImage    *image,
					    const gchar  *filename,
					    const gchar  *format,
					    gpointer      data,
					    gboolean      use_specific_data);
gchar     *gimv_image_detect_type_by_ext   (const gchar  *str);

GList     *gimv_image_get_sysdef_ext_list_all ();
GList     *gimv_image_get_sysdef_ext_list     ();
GList     *gimv_image_get_userdef_ext_list    ();
gchar     *gimv_image_get_type_from_ext       (gchar *ext);


/* FIXME!! */
guchar    *xvpics_load                     (const gchar  *filename,
					    gint         *widthp,
					    gint         *heightp);
GimvImage *gimv_image_get_xv_thumbnail     (const gchar  *thumb_filename);

gboolean   gimv_image_add_layer            (guchar *buffer,
					    gint width,
					    gint left,
					    gint components,
					    gint pass,
					    LayerMode mode,
					    guchar *rgbbuffer);
GimvImage *gimv_image_rgba2rgb             (GimvImage *image,
					    gint       bg_red,
					    gint       bg_green,
					    gint       bg_blue);
/* End FIXME!! */

#endif /* __GIMV_IMAGE_H__ */
