/*
 * GImageView
 * Copyright (C) 2001 Takuro Ashie
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <string.h>

#include "gimageview.h"

#include "plugin.h"
#include "prefs.h"
#include "prefs_ui_plugin.h"
#include "prefs_win.h"

typedef struct PrefsWin_Tag {
   /* plugin */
   GtkWidget *plugin_clist;
   GtkWidget *plugin_entry;
   GtkWidget *plugin_up;
   GtkWidget *plugin_down;
   GtkWidget *plugin_new;
   GtkWidget *plugin_add;
   GtkWidget *plugin_change;
   GtkWidget *plugin_delete;
   gint       plugin_selected;
} PrefsWin;

static PrefsWin  prefs_win;
extern Config   *config_changed;
extern Config   *config_prechanged;


static void
plugin_update_config ()
{
   GtkCList *clist = GTK_CLIST (prefs_win.plugin_clist);
   gint row, can_retrieve = 1;
   gchar *coltext, *string = NULL, *tmpstr;

   for (row = 0; row < clist->rows; row++) {
      can_retrieve = gtk_clist_get_text (clist, row, 0, &coltext);
      if (!can_retrieve) break;

      if (string) {
	 tmpstr = string;
	 string = g_strconcat (string, ",", NULL);
	 g_free (tmpstr);
      } else {
	 string = g_strdup ("");
      }
      tmpstr = string;
      string = g_strconcat (tmpstr, coltext, NULL);
      g_free (tmpstr);
   }

   if (config_prechanged->plugin_search_dir_list
       != config_changed->plugin_search_dir_list)
   {
      g_free (config_changed->plugin_search_dir_list);
   }

   config_changed->plugin_search_dir_list = string;
}


static void
cb_plugin_select_row (GtkCList *clist, gint row, gint col,
		      GdkEventButton *event, gpointer data)
{
   gint success;
   gchar *text[1] = {NULL};

   g_return_if_fail (clist);

   prefs_win.plugin_selected = row;

   success = gtk_clist_get_text (clist, row, 0, &text[0]);
   if (success && text[0])
      gtk_entry_set_text (GTK_ENTRY(prefs_win.plugin_entry), text[0]);

   prefs_plugin_set_sensitive (FALSE);
}


static void
cb_plugin_unselect_row (GtkCList *clist, gint row, gint col,
			GdkEventButton *event, gpointer data)
{
   g_return_if_fail (clist);

   prefs_win.plugin_selected = -1;

    prefs_plugin_set_sensitive (FALSE);
}


static void
cb_plugin_up_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = GTK_CLIST (prefs_win.plugin_clist);
   gint selected = prefs_win.plugin_selected;

   g_return_if_fail (button && clist);

   if (selected < 1 || selected > clist->rows - 1)
   {
      return;
   }

   gtk_clist_swap_rows (clist, selected, selected - 1);
   gtk_clist_moveto (clist, selected - 1, 0, 0, 0);
   prefs_win.plugin_selected--;

   plugin_update_config ();
    prefs_plugin_set_sensitive (FALSE);
}


static void
cb_plugin_down_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = GTK_CLIST (prefs_win.plugin_clist);
   gint selected = prefs_win.plugin_selected;

   g_return_if_fail (button && clist);

   if (selected < 0 || selected > clist->rows - 2)
   {
      return;
   }

   gtk_clist_swap_rows (clist, selected, selected + 1);
   gtk_clist_moveto (clist, selected + 1, 0, 0, 0);
   prefs_win.plugin_selected++;

   plugin_update_config ();
    prefs_plugin_set_sensitive (FALSE);
}


static void
cb_plugin_new_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = GTK_CLIST (prefs_win.plugin_clist);
   /* GtkEntry *entry = GTK_ENTRY (prefs_win.plugin_entry); */

   g_return_if_fail (button);

   gtk_clist_unselect_all (clist);
   /* gtk_entry_set_text (entry, "\0"); */

    prefs_plugin_set_sensitive (FALSE);
}


static void
cb_plugin_add_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = GTK_CLIST (prefs_win.plugin_clist);
   GtkEntry *entry = GTK_ENTRY (prefs_win.plugin_entry);
   const gchar *text[1];

   g_return_if_fail (button);

   text[0] = gtk_entry_get_text (entry);
   gtk_clist_append (clist, (gchar **) text);
   gtk_entry_set_text (entry, "\0");

   plugin_update_config ();
    prefs_plugin_set_sensitive (FALSE);
}


static void
cb_plugin_change_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = GTK_CLIST (prefs_win.plugin_clist);
   GtkEntry *entry = GTK_ENTRY (prefs_win.plugin_entry);
   const gchar *text[1];
   gint selected = prefs_win.plugin_selected;

   if (selected < 0 && selected > clist->rows) return;

   text[0] = gtk_entry_get_text (entry);
   gtk_clist_set_text (clist, selected, 0, text[0]);
   gtk_entry_set_text (entry, "\0");

   plugin_update_config ();
    prefs_plugin_set_sensitive (FALSE);
}


static void
cb_plugin_delete_button_pressed (GtkButton *button, gpointer data)
{
   GtkCList *clist = GTK_CLIST (prefs_win.plugin_clist);
   GtkEntry *entry = GTK_ENTRY (prefs_win.plugin_entry);
   gint selected = prefs_win.plugin_selected;

   if (selected < 0 && selected > clist->rows) return;

   gtk_clist_remove (clist, selected);

   gtk_clist_unselect_all (clist);
   gtk_entry_set_text (entry, "\0");

   plugin_update_config ();
    prefs_plugin_set_sensitive (FALSE);
}


static void
cb_plugin_entry_changed (GtkEditable *entry, gpointer data)
{
    prefs_plugin_set_sensitive (TRUE);
}



/*******************************************************************************
 *  prefs_plugin_page:
 *     @ Create plugin preference page.
 *
 *  Return : Packed widget (GtkVbox)
 *******************************************************************************/
GtkWidget *
prefs_plugin_page ()
{
   GtkWidget *main_vbox, *frame, *frame_vbox, *hbox, *hbox1, *vbox;
   GtkWidget *label, *entry, *scrollwin, *clist, *button;
   GList *list;
   gint i, num;
   gchar *titles[] = {
      N_("Directory"),
   };

   main_vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 5);

   /**********************************************
    *  Search Directory frame
    **********************************************/
   prefs_create_frame (_("Directories list to search plugins"),
		       frame, frame_vbox, main_vbox);

   /* clist */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 0);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, TRUE, TRUE, 0);

   scrollwin = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin),
				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_container_set_border_width(GTK_CONTAINER(scrollwin), 5);
   gtk_box_pack_start (GTK_BOX (hbox), scrollwin, TRUE, TRUE, 0);
   gtk_widget_set_usize (scrollwin, -1, 150);

   num = sizeof (titles)/ sizeof (titles[0]);
   clist = prefs_win.plugin_clist = gtk_clist_new (num);
   gtk_clist_set_column_auto_resize (GTK_CLIST (clist), 0, TRUE);
   for (i = 0; i < num; i++)
      gtk_clist_set_column_title (GTK_CLIST (clist), i, _(titles[i]));
   gtk_clist_column_titles_hide (GTK_CLIST (clist));
   gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_SINGLE);
   gtk_container_add (GTK_CONTAINER (scrollwin), clist);
   gtk_signal_connect (GTK_OBJECT (clist),"select_row",
		       GTK_SIGNAL_FUNC (cb_plugin_select_row),
		       NULL);
   gtk_signal_connect (GTK_OBJECT (clist),"unselect_row",
		       GTK_SIGNAL_FUNC (cb_plugin_unselect_row),
		       NULL);

   /* set default value */
   list = plugin_get_search_dir_list ();
   while (list) {
      gchar *text = list->data;
      gtk_clist_append (GTK_CLIST (clist), &text);
      list = g_list_next (list);
   }

   /* move buttons */
   vbox = gtk_vbox_new (TRUE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);

   button = gtk_button_new_with_label (_("Up"));
   prefs_win.plugin_up = button;
   gtk_widget_set_usize (button, 50, -1);
   gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"released",
		       GTK_SIGNAL_FUNC (cb_plugin_up_button_pressed), NULL);

   button = gtk_button_new_with_label (_("Down"));
   prefs_win.plugin_down = button;
   gtk_widget_set_usize (button, 50, -1);
   gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"released",
		       GTK_SIGNAL_FUNC (cb_plugin_down_button_pressed), NULL);


   /* entry area */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, TRUE, TRUE, 0);

   label = gtk_label_new (_("Directory to add : "));
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   entry = prefs_win.plugin_entry = gtk_entry_new ();
   gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
   gtk_signal_connect (GTK_OBJECT (entry),"changed",
		       GTK_SIGNAL_FUNC (cb_plugin_entry_changed), NULL);

   /* edit buttons */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, TRUE, TRUE, 0);

   hbox1 = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_end (GTK_BOX (hbox), hbox1, FALSE, TRUE, 0);
   gtk_box_set_homogeneous (GTK_BOX(hbox1), TRUE);

   button = prefs_win.plugin_new =gtk_button_new_with_label (_("New"));
   gtk_widget_set_usize (button, 70, -1);
   gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, TRUE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"released",
		       GTK_SIGNAL_FUNC (cb_plugin_new_button_pressed), NULL);

   button = prefs_win.plugin_add = gtk_button_new_with_label (_("Add"));
   gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, TRUE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"released",
		       GTK_SIGNAL_FUNC (cb_plugin_add_button_pressed), NULL);

   button = prefs_win.plugin_change = gtk_button_new_with_label (_("Change"));
   gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, TRUE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"released",
		       GTK_SIGNAL_FUNC (cb_plugin_change_button_pressed), NULL);

   button = prefs_win.plugin_delete = gtk_button_new_with_label (_("Delete"));
   gtk_box_pack_start (GTK_BOX (hbox1), button, FALSE, TRUE, 2);
   gtk_signal_connect (GTK_OBJECT (button),"released",
		       GTK_SIGNAL_FUNC (cb_plugin_delete_button_pressed), NULL);

   prefs_win.plugin_selected = -1;

   return main_vbox;
}


static GtkWidget *
create_plugin_admin_widget (GList *plugin_list)
{
   GtkWidget *hbox, *scrollwin;
   GtkWidget *clist;
   GList *list;
   gint i, num;
   gchar *titles[] = {N_("Plugin Name"), N_("Version"), N_("Module Name")};
   gchar *text[3];

   /* clist */
   hbox = gtk_hbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 0);

   scrollwin = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin),
				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_container_set_border_width(GTK_CONTAINER(scrollwin), 5);
   gtk_box_pack_start (GTK_BOX (hbox), scrollwin, TRUE, TRUE, 0);
   gtk_widget_set_usize (scrollwin, -1, 200);

   num = sizeof (titles)/ sizeof (titles[0]);
   clist = prefs_win.plugin_clist = gtk_clist_new (num);
   for (i = 0; i < num; i++) {
      gtk_clist_set_column_title (GTK_CLIST (clist), i, _(titles[i]));
      gtk_clist_set_column_auto_resize (GTK_CLIST (clist), i, TRUE);
   }
   gtk_clist_column_titles_show (GTK_CLIST (clist));
   gtk_clist_set_selection_mode (GTK_CLIST (clist), GTK_SELECTION_SINGLE);
   gtk_container_add (GTK_CONTAINER (scrollwin), clist);

   /*
   gtk_signal_connect (GTK_OBJECT (clist),"select_row",
		       GTK_SIGNAL_FUNC (cb_plugin_select_row),
		       NULL);
   gtk_signal_connect (GTK_OBJECT (clist),"unselect_row",
		       GTK_SIGNAL_FUNC (cb_plugin_unselect_row),
		       NULL);
   */

   /* set default value */
   list = plugin_list;
   while (list) {
      GModule *module = list->data;

      text[0] = _(plugin_get_name (module));
      text[1] = _(plugin_get_version_string (module));
      text[2] = _(plugin_get_module_name (module));

      gtk_clist_append (GTK_CLIST (clist), text);

      list = g_list_next (list);
   }

   return hbox;
}



/*******************************************************************************
 *  prefs_plugin_image_saver_page:
 *     @ Create image saver plugin preference page.
 *
 *  Return : Packed widget (GtkVbox)
 *******************************************************************************/
GtkWidget *
prefs_plugin_image_saver_page ()
{
   GtkWidget *main_vbox, *frame, *frame_vbox, *hbox;
   GList *list;

   main_vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 5);

   /**********************************************
    *  Plugin list
    **********************************************/
   prefs_create_frame (_("Image Saver Plugins"), frame, frame_vbox, main_vbox);

   /* clist */
   list = plugin_get_list (PLUGIN_IMAGE_SAVER);
   hbox = create_plugin_admin_widget (list);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, TRUE, TRUE, 0);

   return main_vbox;
}



/*******************************************************************************
 *  prefs_plugin_archive_page:
 *     @ Create archive plugin preference page.
 *
 *  Return : Packed widget (GtkVbox)
 *******************************************************************************/
GtkWidget *
prefs_plugin_archiver_page ()
{
   GtkWidget *main_vbox, *frame, *frame_vbox, *hbox;
   GList *list;

   main_vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 5);

   /**********************************************
    *  Plugin list
    **********************************************/
   prefs_create_frame (_("Archiver Plugins"), frame, frame_vbox, main_vbox);

   /* clist */
   list = plugin_get_list (PLUGIN_EXT_ARCHIVER);
   hbox = create_plugin_admin_widget (list);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, TRUE, TRUE, 0);

   return main_vbox;
}



/*******************************************************************************
 *  prefs_plugin_thumbnail_page:
 *     @ Create thumbnail plugin preference page.
 *
 *  Return : Packed widget (GtkVbox)
 *******************************************************************************/
GtkWidget *
prefs_plugin_thumbnail_page ()
{
   GtkWidget *main_vbox, *frame, *frame_vbox, *hbox;
   GList *list;

   main_vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(main_vbox), 5);

   /**********************************************
    *  Plugin list
    **********************************************/
   prefs_create_frame (_("Thumbnail Cache Plugins"),
		       frame, frame_vbox, main_vbox);

   /* clist */
   list = plugin_get_list (PLUGIN_THUMB_CACHE);
   hbox = create_plugin_admin_widget (list);
   gtk_box_pack_start (GTK_BOX (frame_vbox), hbox, TRUE, TRUE, 0);

   return main_vbox;
}


void
prefs_plugin_set_sensitive (gboolean text_changed)
{
   GtkCList *clist = GTK_CLIST (prefs_win.plugin_clist);
   GtkEntry *entry = GTK_ENTRY (prefs_win.plugin_entry);
   gint selected = prefs_win.plugin_selected;
   const gchar *text;

   text = gtk_entry_get_text (entry);

   if (selected < 0) {   /* no item selected */
      gtk_widget_set_sensitive (prefs_win.plugin_up, FALSE);
      gtk_widget_set_sensitive (prefs_win.plugin_down, FALSE);
      gtk_widget_set_sensitive (prefs_win.plugin_change, FALSE);
      gtk_widget_set_sensitive (prefs_win.plugin_delete, FALSE);
   } else {
      /* up button */
      if (selected == 0)
	 gtk_widget_set_sensitive (prefs_win.plugin_up, FALSE);
      else
	 gtk_widget_set_sensitive (prefs_win.plugin_up, TRUE);

      /* down button */
      if (selected < clist->rows - 1)
	 gtk_widget_set_sensitive (prefs_win.plugin_down, TRUE);
      else
	 gtk_widget_set_sensitive (prefs_win.plugin_down, FALSE);

      /* change button */
      if (text_changed)
	 gtk_widget_set_sensitive (prefs_win.plugin_change, TRUE);
      else
	 gtk_widget_set_sensitive (prefs_win.plugin_change, FALSE);

      /* delete button */
      gtk_widget_set_sensitive (prefs_win.plugin_delete, TRUE);
   }

   if (!text || *text == '\0')
   {
      gtk_widget_set_sensitive (prefs_win.plugin_new, FALSE);
      gtk_widget_set_sensitive (prefs_win.plugin_add, FALSE);
      gtk_widget_set_sensitive (prefs_win.plugin_change, FALSE);
   } else {
      gtk_widget_set_sensitive (prefs_win.plugin_new, TRUE);
      gtk_widget_set_sensitive (prefs_win.plugin_add, TRUE);
      if (selected >= 0)
	 gtk_widget_set_sensitive (prefs_win.plugin_change, TRUE);
   }
}
